//CA.HPP=============================================

//=================COMPILER SWITCHES==================================
#ifndef CA_HPP
#define CA_HPP

#include "types.h" //To set FLAT right away

#ifdef __FLAT__
/* In the 32 bit model you have the optoin of using a Windows
built in TOOLBAR.  This is presently commented out as it is broken */
	#define TOOLBAR
	#define TOOLBARHEIGHT 33
#else
	#define TOOLBARHEIGHT 0
#endif //__FLAT__

//#define LITE
 /*Allow only 1 CA.  When this is on, we call the
build CAPOWLOW.EXE (instead of CAPOW.EXE).  There is another build option
in types.h, we can define DOUBLE or not, and build CAPOWDBL.EXE or
CAPOWDLO.EXE, if we wanted to combine LITE and DOUBLE.*/
#ifdef LITE
#define FIXED_640_480 //Control this being in or not with LITE.
/* Allow only a maximum bitmap size of 640 by 480.  This lowers the
memory requirements by (a) making the _max_horz_count lower in the CAs
and, more important by lowering the size of the CX and CY in your
WindowBitmap WBM.  Ordinarily I turn this on just when LITE is on.*/
#endif//LITE

#define NO_DEFAULT_COL /*Comment out some stuff that should be TODO erased.*/
//#define FIXED_FREQ //incorrect attempt to fix global freq at 1.0
//#define ADHOC_FREQ_FIX  //in CA::Adjust_acceleration_multiplier
#define FIXED_CELL_FREQ   //don't vary the freq in the cells
//#define TEMP_COLOR_FIX
	/*The fix is because colors were no longer being loaded properly,
	so says go ahead and randomize the color each time we load.
     TODO is this still needed?*/
#define NO_FARMALLOC
/* I *think* that the C++ new type[] operator will not create huge arrays.
So if I need a huge array --- this happens if the 2D CA waveplane buffers
get large --- then I probably need to turn the NO_FARMALLOC switch off
and go back to using the Borland specific farmalloc for that one
allocation.  Otherwise we prefer to avoid farmalloc and use new instead.*/

#define EDGE_FLUTTER_2D
/*If this is off, then when I make wrapflag FALSE for a 2D CA with
blankedgeflag FALSE, I set all the edge cells past,source,and target
intensities to the same value.  If this is on, then I don't set these
values to the same value, which makes the edges "flutter", which makes
an interesting effect like an ocean edge.  See CA::SetWrapFlag.*/

#define SMOOTH_VARIANCE
/* This is used by CA::Smooth_variance.  The idea is to smooth out the
varying parameters between neighboring cells.*/

#define FAST_ENTROPY
/* This means to use a faster formulat in CA::Entropy*/

//#define ONE_AT_A_TIME
/*Used in CAlist::Update_and_Show.  This make a faster response to the
controls, by only doing one CA update on each pulse of CellMain.  It seems
to to slow the runspeed down by too great a factor*/

//#define TAXI_CAB_METRIC
/*Used in seed.cpp in CA::OneSeed for the dimension == 2 case.  If this
is in, I use the abs(x)+abs(y) metric for sloping up to the seed point.
If this is out, then I use the slightly slower but far superior
standard Euclidean metric to get a nice cone of seed */

//#define RUNGE_KUTTE
/* This was for an alternate approach to the Euler integration used
in particular by the oscillator methods.  But Ostrov said that EULER
should work as well as RUNGE_KUTTE for our small _dt, and in fact he
seems to be right.*/

#define SEED_VELOCITY
/* Moving on to the reaction diffusion rules, we want to use the
Wavecell velocity field as an independent variable, so we need to
seed this.  This switch is used in SEED.CPP */

//=================INCLUDES==================================
#include <fstream.h> //For iostream
#include "bitmap.hpp" //For WindowBitmap class
#include "dllmem.hpp"  //For DLLlist class
#include "tweak.hpp"   //For TweakParam class
//============MACROS============================================
#define MIN(x,y) ((x)<=(y))?(x):(y)
/* We use CLAMP on Reals in the CA rules, also we use it on ints and
floats in the accessors to set various params*/
#define CLAMP(x,lo,hi) (x)=(((x)<(lo))?(lo):(((x)>(hi))?(hi):(x)))
/*When you clamp an unsigned short colorindex, you don't have to
worry about it being negative*/
#define POSITIVECLAMP(x,hi) (x)=(((x)>(hi))?(hi):(x))
/*In the Ulam rules, we want to do a CLAMP and know if it happened, so
we need this variation, which we write as inline instead of macro.*/
inline BOOL RealClampAndTell(Real &x, Real lo, Real hi)
{
	if (x < lo)
	{
		x = lo;
		return TRUE;
	}
	if (x > hi)
	{
		x = hi;
		return TRUE;
	}
	return FALSE;
}
/*This could be a macro, but is too long, so we make it a "macro" by
inline-ing it*/
inline void WRAP(Real &vx, Real vlo, Real vhi)
 //  {if (vx<vlo){vx = vlo;return;} if (vx>vhi)vx = vhi;return;}
{   //assyne vl0 = -vhi, and vhi > 0.
	if (vx<vlo)
	{
		if (vx < vlo + vlo)    //vlo is negative
			vx = vlo;
		vx = vhi - (vlo-vx);
		return;
	}
	if (vx>vhi)
	{
		if (vx > vhi + vhi)
			vx = vhi;
		vx = vlo + (vx - vhi);
	}
	return;
}
//=========================DEFINES=================================
//-------------------------GENERAL ADJUSTMENT DEFINES--------------------
#define MAXFILENAME 256 //Length of file names
#define BORDER 5 //Spacing for the lines between CAs.
#define FILEVERSION 7
#ifndef LITE
#define MAX_CAS 9 /*64 works, but use less for faster comeup.  With
	the advent of 2D CAs, we also are using less of these.  For building
	LITECPW for low-memory machines, set this to 1*/
#define START_CAS 9
#else //LITE
#define MAX_CAS 1 /*  For building
	LITECPW for low-memory machines, set this to 1*/
#define START_CAS 1
#endif //LITE
//-------------------------2D ADJUSTMENT DEFINES----------------------
#ifndef DOUBLE
#define CX_2D 120
#define CY_2D 90
#else //DOUBLE
/*If you use DOUBLE then you use twice as much memory per cell,
and this runs out of memory reallly easy */
#define CX_2D 10
#define CY_2D 10
#endif //DOUBLE
/*If this product is less than 64K/sizeof(Real) we can use
far instead of huge*/
//#define HUGE_2D
#ifdef HUGE_2D
typedef unsigned long index2D_type;
#else //HUGE_2D
typedef unsigned int index2D_type;
#endif //HUGE_2D
//--------------------TWEAKPARAM DEFINES--------------------
	/* These have been moved to tweakca.hpp */
//-------------------------WAVE ADJUSTMENT DEFINES--------------------
#define ALTSMOOTHSTEPS 3
//Number of steps of heat rule to run after a discontinuity is
//introduced by seeding or by resizing or by toggling wrapflag.
#define MINSMOOTHSTEPS 3 //Don't use 1 or 2 or bad things may happen.
#define SMOOTHRANDSTEPSFACTOR 3 //Smooth 3 time as much for RandomSeed.
#define SMOOTHEDGESTEPS 1000 //this is used as a flag, you only do ONE step.
#define SMOOTHEDGERADIUS 20 /*How many pixels to smooth on either side of a
	changed edge*/
#define WAVE_PERIOD 200
#define GENERATOR_PERIOD 200
#define OSCILLATOR_BASE_PERIOD 200 //This  many timesteps
#define BLT_LINES 3 //4 is faster, but I don't like the way it looks
	//Used to chunk lines to put on screen.
//--------------------DIGITAL ADJUSTMENT DEFINES---------------
#define STATERADIUSCOUNT 10
#define MAXNABEOPTIONS 0x4000U //16K
#define MEMORY 16 /*This must be >= 2.  You need to be careful if it
	ever exceeds horz_count or vert_count.  This tracks how many
     past rows you remember for StripeAvoid.  Applies mainly to digital
	  CAs.*/
#define MAX_STATES 32
#define MAX_RADIUS 7
#define HIGH_FITNESS (Real)0.9
#define MAX_LAMBDA (Real)0.90
#define MIN_LAMBDA (Real)0.10
//-------------------- MIN MULTIPLIERS -----------
#define GENERAL_MIN_MULTIPLIER (Real)0.0003
#define NONLINEARITY_MIN_MULTIPLIER (Real) 0.01
#define DRIVER_MIN_MULTIPLIER (Real) 0.1
#define MIN_POS_CHUNK 0.001
//--------------------EXPERIMENT DIALOG DEFINES-----------
#define RAMP_X_MIN (Real)0.1
#define RAMP_X_MAX (Real)0.3
#define RAMP_Y_MIN (Real)0.1
#define RAMP_Y_MAX (Real)0.3
	// used for experiment dialog
//-------------------ENUMS DEFINES FOR CONTROLS-------------
#define BOTH_SHOW 0
#define ODD_SHOW 1
#define EVEN_SHOW 2
#define ZAP 0	// breed methods
#define CLONE 1
#define SEX 2
#define NONE 3
#define MAX_EVOLVEFLAG 4
#define CA_STANDARD 10
#define CA_REVERSIBLE 11
#define CA_WAVE 12
#define CA_WAVE2 13
#define CA_HEATWAVE 14
#define CA_OSCILLATOR 15
//#define CA_OSCILLATOR_WAVE 16
#define CA_USER 16
#define CA_DIVERSE_OSCILLATOR 17
#define CA_DIVERSE_OSCILLATOR_WAVE 18
#define CA_ULAM_WAVE 19
#define CA_AUTO_ULAM_WAVE 20
#define ALT_CA_WAVE 21
#define ALT_CA_WAVE2 22
#define ALT_CA_OSCILLATOR 23
#define ALT_CA_OSCILLATOR_WAVE 24
#define ALT_CA_DIVERSE_OSCILLATOR 25
#define ALT_CA_DIVERSE_OSCILLATOR_WAVE 26
#define ALT_CA_ULAM_WAVE 27
#define ALT_CA_AUTO_ULAM_WAVE 28
#define CA_HEATWAVE2 29
#define CA_WAVE_2D 30
#define CA_HEAT_2D 31
#define MAX_TYPE_CA 23 //counts number of types in use.
//---------------------START VALUE DEFINES------------
#define START_TYPE_CA ALT_CA_WAVE//CA_WAVE_2D needs START_VIEWMODE IDC_2D_VIEW
#define START_SHOWMODE BOTH_SHOW
#define START_VIEWMODE IDC_SPLIT_VIEW //param defined in camenu.h
#define START_SHOWVELOCITY FALSE
#define START_WRAPFLAG  TRUE
#define START_BLANKEDGEFLAG TRUE
#define START_ENTROPYFLAG 1
#define START_EVOLVEFLAG ZAP
#define START_MUTATE_FLAG 0
#define START_TARGET_ENTROPY (Real)0.5
#define START_FAILSTRIPE_SCORE -100
#define START_ENTROPY_BONUS 200
#define START_BREED_FLAG 0
#define START_SEED_FLAG	0
#define START_BREED_CYCLE	600
#define START_SEED_CYCLE	400
#define START_SEED_METHOD	1
#define START_LAMBDA (Real)0.3
#define START_MUTATION_STRENGTH (Real)0.10
#define MAX_GENERATOR_INTENSITY (Real)256.0
#define START_BAND_COUNT 16
#define START_MIN_NONLINEARITY3 (Real)1.0
#define START_MAX_NONLINEARITY3 (Real)200.0
#define START_RADIUS 1
#define START_STATES 8
#define START_SCROLL_FLAG 	0 //0 is off, 1 is on
#define START_STRIPEKILL_FLAG 0
#define START_STRIPESEED_FLAG 0
#define START_GENERATOR_FLAG 0
#define START_COLORRAMP_FLAG 0
#define START_GENERATOR_LOCATION 0
#define START_WIRE_FLAG 0
#define START_GRAPH_FLAG 0
//-----------------------COLOR DEFINES--------------------------
// number of anchor colors in the logical palette
#define MAX_COLOR_LOGIC 13
#define MAX_COLOR 1024U //colorindex ranges over this
#define MAX_BAND_COUNT 128
#define DEFAULT_BAND_COUNT 13
#define BLACK RGB(0,0,0)
#define WHITE RGB(255,255,255)
#define START_COLORRAMP_FLAG 0
#define SPLIT_SCROLL_PROPORTION 0.7
//------------------FOURIER DEFINES-------------------- Bang-Nguyen
#define MAXTERM      100  	// Maximum # of terms in the Fourier series
#define TPMAXARRAY  4000   // Maxximum size of the test-point array
//------------------COMPRESS DEFINES----------------------
#define  table_size   70
#define  s_size       150
#define  htable_size   16
#define  SIGNATURE    "capow_codes"
//--------------------CELL PARAM DEFINES------------------
#define CELL_PARAM_COUNT 3 //Number of extra cell params.
//Right now I actually only need three extra for friction,
//spring and mass a.k.a resistance, capacitance, induction.
#define friction_tweak _cell_param[0]
#define spring_tweak _cell_param[1]
#define mass_tweak _cell_param[2]

//=======================GLOBAL FUNCTION DECLARATIONS=========
extern BOOL timer_flag;	// Take a time reading? 1 - yes
extern BOOL update_flag; // Update all the dialog boxes? 1 - yes
	/*These live in CA.HPP */

//=========================WAVECELL CLASS==========================
struct Wavecell  //a struct is an all-public class
{
	COLORREF state;
	Real intensity;
	Real velocity;
	Real _cell_param[CELL_PARAM_COUNT];
public:
	Wavecell();
 };

 struct Wavecell2  //a struct is an all-public class
{
	Real intensity;
public:
	Wavecell2():intensity(0.0){};
 };

//============================GENERATOR CLASS==============================
class Generator
{
	friend class CA;
	friend class CAlist;

private:
	CA *ca_ptr; /* know your owner */
	int location; /* know the x coordinate of your onecell location
		relative to the row buffer of your owner*/
	unsigned char state;
	Real intensity; /* for doing waves */
	Real velocity;
	Real time; /* ticks of time increment in timestep of myca.  At
		present (Rudy November 7, 1994) we are using velocity_
		mulitplier for the timestep. */
public:
	Generator(CA *myca); //constructor.  We use the default destructor.
	void Seed(){intensity = 0.0; time = 0.0; state = 0;}
	void Step();
};

//==============DERIVPAIR CLASS==========================
#ifdef RUNGE_KUTTE
struct DerivPair
{
	Real _di;
	Real _dv;
	DerivPair(){}
	DerivPair(Real di, Real dv):_di(di),_dv(dv){}
};
#endif //RUNGE_KUTTE


extern "C" DllExport USERRULE(CA FAR *, int, int, int);

//============================CA CLASS==============================
/* In the comments on the fields of the CA class, we note which
params are AUTOSET by some method, and which are to be set by LOADSAVE
values which you can read and write to files.*/
class CA
{
	friend class CAlist;
	friend class Generator;
	friend ofstream& operator<<(ofstream& ofs, CA* target);
	friend ifstream& operator>>(ifstream& ifs, CA* target);
	friend void Version_123_Load_Individual(ifstream& ifs, CA* target,
		int version); // DDC 4-25-96
	friend void Version_45_Load_Individual(ifstream& ifs, CA* target); // DDC 4-25-96
	friend void Version_6_Load_Individual(ifstream& ifs, CA* target); // DDC 4-25-96
	friend void Version_7_Load_Individual(ifstream& ifs, CA* target); // DDC 4-25-96
private:
//==============================CA Private=========================
	WindowBitmap *WBM; //AUTOSET by CAlist::SetWindowBitmap
	HWND hwnd; //AUTOSET by CAlist::CAlist.Where I live.
 	class CAlist *calist_ptr; //AUTOSET by CAlist::CAlist.  The parent.
	int type_ca; //LOADSAVE set, used by CA::Settype.
	int _phylum; /* NEW LOADSAVE, used by CA::Settype when type_ca is
		CA_USER */
	int dimension; //AUTOSET by CA::Settype. 1D or 2D.
	int viewmode; /* LOADSAVE.  Currently IDC_DOWN_VIEW, IDC_SCROLL_VIEW,
		IDC_WIRE_VIEW IDC_GRAPH_VIEW, IDC_SPLIT_VIEW, IDC_2D_VIEW*/
	int row_number; /* AUTOSET. Initialized in CA::Locate and updated
     	in CA::?????Update.  A counter to keep track where you are.
		This is wrapped back to miny after it hits maxy. */
	long int cellcount; /*AUTOSET. Initialized in CA::Locate and updated
     	in CA::?????Update. How many cells have you updated so far?  Use
		this with entropy and freqlookup. */
	BOOL wrapflag; /* LOADSAVE.  Set by CAlist::SetWrapFlag, not by
     	CA::SetWrapFlag, because you need to set the smooth param as
          well.  TRUE means wrap and FALSE means don't.*/
	BOOL blankedgeflag;/* LOADSAVE. TODO: implement the accessors.
     	TRUE means that when you set wrapflag to FALSE, you set the edges
          to 0.0 values.  FALSE means you leave whatever crud in the edges.*/
//-------------------CA Color Private-----------------------------
	int band_count; //LOADSAVE used by all continuous CAs
	COLORREF far *colortable; /*AUTOSET by CA::Setcolortable.
		For digital CAs, the color of a cell will just be
          colortable[state].  For CA_wave guys, we'll do some map from
          intensity into the MAX_COLOR range to get colorindex_target_row,
          and use colortable on that.*/
//---------------------CA Window Private-----------------
	int _max_horz_count; //AUTOSET Either screensize or 640, see FIXED_640_480 switch.
   int horz_count; /*LOADSAVE.  This measures the active visible length
     	of the 1D CA.  TODO, when you load, use this number
     	to force the window to the desired size.*/
	int minx, miny, maxx, maxy, vert_count; /* AUTOSET
		by CAlist::Locate and CA::Locate according to tile_number and
          the size of the display */
	int splity, split_vert_count; /*AUTOSET by CAlist::Locate.
		For IDC_SPLIT view with SCROLL on top, GRAPH on bottom.
          splity is somewhere between miny and maxy; split_vert_count
          measures maxy - splity, the space for the graph.*/
	int tile_number; /*AUTOSET by CAlist::Locate.  Use this for
     	multiple CAs on one screen.*/
//---------------------CA Generator Private-----------------
	BOOL generatorflag; /*LOADSAVE.*/
	class Generator *generator_ptr; /*AUTOSET by CA::CA.  TODO, make
		this a list so you can have multiple genearators.*/
//---------------------CA Evolution Private-----------------
	Real fitness; /* AUTOSET. lies between 0 and 1, is typically set to
		fabs(entropy - target_entropy). But if a rule fails the
		shift test then its fitness is 0. */
	BOOL entropyflag; /* LOADSAVE. 1 means compute entropy every 64K
		cells (and act on it) and 0 means don't */
	Real entropy; /* AUTOSET. normalized entropy lies between 0.0 and 1.0, it
		tells what percentage of maximum entropy the nabe patterns have.*/
	Real target_entropy; /* LOADSAVE. normalized entropy between 0.0 and 1.0,
		that you want to evolve towards */
	Real entropy_bonus;
		/* LOADSAVE. The score bonus for being close to targ4et_entropy */
	Real fail_stripe; /* LOADSAVE. score penalty for fail strip-ing */
	Real score; /* AUTOSET. A ca's current score */
	unsigned short far *freqlookup; //AUTOSET
		/* freqlookup keeps track of how often each nabeoption occurs.
		Whenever you check lookup[nabecode], you at the same time
		increment freqlookup[nabecode].  Note that you can run ths over
		64K really quickly, like in 100 lines of fullscreen.  But suppose
		we only use in screens of 4 or 9 CAs.  In 4, it takes 200, not
		quite a full frame.  Well, that'll do.  And I'll check entropy
		each time the worst case count hits 64K.  unsigned char would
		be out of the question, like one line even in 9 mode.  Would
		prefer long unsigned int, kind of, but that's overkill.  At
		the worst, counting lookup and freqtalbe, we are looking at
		9*3*32K  = 864K bytes, which won't work.  So there has to be
		a check on it.*/
//-------------------Fourier CA private-------------------
	int	test_point, temp_test_point;
	Real	rel_test_point;
	Real 	*tp_real_array;
	Real 	*fourier_a,     			// to contain the Fourier coefficients
			*fourier_b;             //
	Real 	*fourier_approx;			// approximating values by Fourier series
	int numofterm;        		// number of terms in the approximation
	int   tp_startpos, tp_endpos;
	BOOL  fourierflag;      		// fourier stuff being displayed or not
	int	tp_viewmode;				// time scale 1/2, 1, or 2
	int tp_viewtype;				// graph or scroll
	int tp_approxtype;				// sine only, cosine only, or both
	BOOL	tp_graphflag, tp_approxflag,
			tp_cosineflag, tp_sineflag, tp_spectrumflag;
//--------------CA--Tweak Params Private-------------------------
	//....................CA.Wave Tweak Params...............
	AdditiveTweakParam _max_intensity;  // LOADSAVE
	AdditiveTweakParam _max_velocity;  // LOADSAVE
	MultiplicativeTweakParam _chunk; //LOADSAVE.
	MultiplicativeTweakParam _dx;  // LOADSAVE
	MultiplicativeTweakParam _dt;  // LOADSAVE
	//....................CA.Wave Tweak Params...............
	//----Oscillator--------
	MultiplicativeTweakParam _frequency_multiplier; //LOADSAVE
	MultiplicativeTweakParam _driver_multiplier; //LOADSAVE
	MultiplicativeTweakParam _spring_multiplier; //LOADSAVE
	MultiplicativeTweakParam _mass;  // LOADSAVE
	MultiplicativeTweakParam _friction_multiplier;
	//....................CA Diverse Tweak Params...............
	MultiplicativeTweakParam _variance;//LOADSAVE
	//....................CA Heat Tweak Params...............
	MultiplicativeTweakParam _heat_inc;
 		/* LOADSAVE.  This is the amount to increment the intensity*/
	//....................CA Nonlinear Tweak Params...............
	MultiplicativeTweakParam _nonlinearity1, _nonlinearity2; /*LOADSAVE.
		nonlinearity1 is used by fermi-pasta-ulam, and
		nonlinearity2 is used by the cubic ferm-pasta-ulam.*/
	//....................CA Not Yet Tweak Params...............
	Real  _phase; //LOADSAVE, Used by Oscillator types.
	Real lambda; /* LOADSAVE. Used by Standard and Reversible.  This
		measures the density of non-zero	entries desried in the lookup table.
		Start with START_LAMBDA.*/
//-------------------CA Userrule private--------------------------------
	char _userrulename[MAXFILENAME];
	FPCELLUPDATE _lpfnUSERRULE; // typedef of FPCELLUPDATE is in types.h 
//---------------CA Digital Private----------------------------------------
	int sourcerowindex; /* AUTOSET. always have source_row =
		rowbuffer[sourcerowindex] */
	int targetrowindex; /* AUTOSET. always have target_row =
		rowbuffer[targetrowindex].  Also keep targetrowindex =
		sourcerowindex + 1 MOD MEMORY.*/
	unsigned char far *source_row;//LOADSAVE.  This is TODO.
	unsigned char far *target_row;
		/* use these two buffers to update the CA; the present values are
		read from source_row and the new values are stored in targdt_row.
		Then you swap these two pointers, show the source_row, and do the
		update again.  These are allocated in Locate, after the actual
		size of the CA is known.*/
	unsigned char far *rowbuffer[MEMORY]; /*AUTOSET. Use for remembering
     	old rows and checking for repeats stripes.*/
	int radius; //LOADSAVE. radius of cells to examine, range 1 to 4.
	int states; //LOADSAVE. number of states per cell.
	int statebits; //AUTOSET. number of bits per state.
	int nabesize; /* LOADSAVE 1 + 2*radius, the number of cells in a
     	neighborhood.*/
	unsigned short nabeoptions; //AUTOSET. Keep this under 32K
		/* pow(2,nabesize), the number of possible lookup
		tables.  For radius 1,2,3,4, this is 8,32,128,512. */
	unsigned short mask; /* AUTOSET.  nabeoptions - 1. Use to mask the
     	nabecode in Update.*/
	Real actual_lambda; /* AUTOSET.  actuallambda measures the density
     	of non-zero entries that are actually in the lookup table.
          This needs to be kept separate from the *desired* value
          lambda because the value drifts in a brownian walk, and can
          end up stuck at 0.0 or 1.0 unless you keep normalizing it to
			 the desired value lambda.*/
	unsigned char far *lookup;/* LOADSAVE.  lookup holds the new cell
     	values, indexed by the nabe code numbers which are assembled in
          Update.  lookup is allocated in the constructor.  Lookups are
          all well under 32K, so don't need huge.*/
//------------------CA Reversible Digital private-------------------------
	int pastrowindex; //AUTOSET
	unsigned char far *past_row; //LOADSAVE.  This is TODO
//-----------------------CA Wave private----------------------------------
	int monochromeflag; //NEW_LOADSAVE
	int _startsmoothsteps; /* default startsmoothsteps is SMOOTHSTEPS for
		the Alt??? type rules, 0 for the others */
	int _smoothsteps; /*AUTOSET. Use this as the number of remaining steps
		of heat rule to run in order to eliminate discontinuities,
		this gets set to startsmoothsteps after OneSeed, Seed, or Locate.
		Need this for the Alt???? type rules. */
	BOOL _justloadedcells; /*Set this to TRUE when you have just loaded a
		rule which loads individual cell values, this is to prevent the
		call to smooth which would wipe out the values */
	Real _stretch_oldmaxintensity;/*AUTOSET. This and the following
     	_stretch_??? params are all helper variables used in the smooth
			 process.  Search for "smoothsteps--" in WaveUpdate and WaveUpdate2D
          CA.CPP to see them in action.  Note that I don't need to initialize
          them in the CA::CA constructor; they get initialized in
          WaveUpdate and WaveUpdate2D*/
	Real _stretch_newmaxintensity;//AUTOSET.
	Real _stretch_oldminintensity;//AUTOSET.
	Real _stretch_newminintensity;//AUTOSET.
	double _stretch_oldavgintensity;//AUTOSET.
	double _stretch_newavgintensity;//AUTOSET.
	BOOL _stretch_lasttime;//AUTOSET.
	double _stretch_cell_weight;//AUTOSET.
	BOOL _dx_lock; /* LOADSAVE. Whether or not user can change _dx to value
     	 other than 2.0 * _dt.  TRRE means no other vals, FALSE means
           yes you can.*/
	int showmode;  /*LOADSAVE.  This determines whether you show
     	all the cells or just every other cell.*/
	int showvelocity; /*LOADSAVE.  0 means show intensity, 1 means
     	show velocity.*/
	Real time; /* LOADSAVE.  I don't think we use it really, but at some
     	point it might make a difference what the CA thinks the time
          is, particularly if this time gets used by the generators.*/
	  Real _wavespeed, _dt_2, _dx_2, _dt_over_2, _dt_over_6, _dt_over_dx_2,
		_wavespeed_2_times_dt_2_over_dx_2, _dt_over_12_times_dx_2,
		_wavespeed_2_times_dt_2_over_12_times_dx_2,_1_over_mass,
			 _dt_over_mass, _wavespeed_2_times_dt_over_dx_2;
          /* AUTOSET. _wavespeed is always 1.0 at this point.  All the other
			 params get set by CA::Adjust_acceleration_multiplier() on
			 the basis of _dt, _dx, _dx_lock, and _wavespeed.*/
	Wavecell far *wave_past_row;  //LOADSAVE (TODO). Holds info for time - 1
	Wavecell far *wave_source_row; //LOADSAVE (TODO) Holds info for time
	Wavecell far *wave_target_row; //AUTOSET. Holds infor for time + 1
	COLORREF far *COLORREF_target_row; /* AUTOSET. Convert digital
         	target_row by doing lookup into colortable, or convert Wave
     	colorindex_target_row by	doing lookup.  The difference is that
     	target_row is for digital states which will only be one byte,
     	but Wave's colorindex_target_row indices are two bytes.*/
	int wavepastindex; /* AUTOSET. always have wave_old_row =
		waverowbuffer[waveoldindex] */
	int wavesourceindex; /* AUTOSET. always have wave_source_row =
		waverowbuffer[wavesourceindex] */
	int wavetargetindex; /* AUTOSET. always have wave_target_row =
		waverowbuffer[wavetargetindex] */
	Wavecell far *waverowbuffer[3];//AUTOSET.
	unsigned short far *colorindex_target_row; /* AUTOSET. Use this
     	instead of target_row to save the (possibly bigger than 256)
     	color index values for wave.*/
	COLORREF far *_anchor_color; //LOADSAVE.
//---------------CA Wave 2D private-----------------------------------
	int horz_count_2D, vert_count_2D; /*LOADSAVE.  TODO as with
		the 1D horz_count, these should be forced in by making the
          window the right size.  ACtually you don't need to change
			 the windwo size if you use stretchblt.*/
	int maxx_2D, maxy_2D; /* LOADSAVE.  These are set according to
     	available memory for the buffer.  At present we AUTOSET them,
          but eventually it would be better to allocate.  If in Locate
          we find that the horz_count_2D, say is larger than the maxx_2D,
			 then we use the smaller value and show with StretchBlt.*/
#ifdef HUGE_2D
	Wavecell2 huge *wave_past_plane;  //LOADSAVE (Maybe TODO). time - 1
	Wavecell2 huge *wave_source_plane; //LOADSAVE (Maybe TODO). time
	Wavecell2 huge *wave_target_plane; //AUTOSET. Holds infor for time + 1
	Wavecell2 huge *waveplanebuffer[3];
#else
	Wavecell2 far *wave_past_plane;  //LOADSAVE (Maybe TODO). time - 1
	Wavecell2 far *wave_source_plane; //LOADSAVE (Maybe TODO). time
	Wavecell2 far *wave_target_plane; //AUTOSET. Holds infor for time + 1
	Wavecell2 far *waveplanebuffer[3];//AUITOSET
#endif //HUGE_2D
	 /* .  We always have
		wave_old_plane = waveplanebuffer[waveoldindex]  always have
			 wave_source_plane = waveplanebuffer[wavesourceindex]  always have
			 wave_target_plane = waveplanebuffer[wavetargetindex] */
//------------------CA Wave Oscillator private-----------------------------
	Real frequency_factor; /*AUTOSET.  This is
	  is a helping variable defined in CA::Adjust_acceleration as
		2*PI/(_dt*OSCILLATOR_BASE_PERIOD), so that BASE_PERIOD
		steps puts it thru a full oscillation.*/
public:
//=========CA====CONSTRUCTORS===================================
	CA(CAlist *mylist);
		/* Sets Allocates. constructor is called when a CA object is
		 declared.  I have the constructor set all the possible fields.
		adopts the hwnd of mylist, adopts mylist as calist_ptr.*/
	~CA(); /* The destructor.  Frees the memory in source_row,
		target_row, lookup, etc.*/
	void Allocate();
		/*Allocates rowbuffer[i], lookup and freqlookup  handles*/
//==============ACCESSORS=====================================
	int Gettype(void) {return type_ca;}
	int Getshowmode(){return showmode;}
	BOOL Getwrapflag(void) {return wrapflag;}
	BOOL Getblankedgeflag(void) {return blankedgeflag;}
	int Getdimension(){return dimension;}
//------------------CA-Color accessors-------------------------------
	COLORREF far *GetColortable() { return colortable;} //used to be huge
	COLORREF far *GetAnchors() { return _anchor_color;}
	COLORREF Color_look(int numb);
		// Returns colortable[lookup[numb]]. Used by Lookup Dialog Box???
	int Get_monochromeflag(){return monochromeflag;}
	int Getband(void) {return band_count;}
//---------------------CA-TweakParam Accessors---------------------------
	TweakParam* GetTweakParam(int tweakParamType);
//---------CA-Digital Accessors--------------------------------------
	int Getradius(void) {return radius;}
	int Getstates(void) {return states;}
	unsigned short Getnabe(void) {return nabeoptions;}
		// Returns the number of nabeoptions. Used by Lookup Dialog Box
	Real Getlambda(void) {return lambda;}
	Real Getactuallambda(void) {return actual_lambda;}
//---------------------CA-Wave accessors-------------------------------
	int Get_startsmoothsteps(){return _startsmoothsteps;}
	Real Get_phase(void) {return _phase;}
	BOOL Get_dx_lock(){return _dx_lock;}
	int Getshowvelocity(){return showvelocity;}
//----------------CA-Evolution accessors--------
	int Getentropyflag(void) {return entropyflag;}
	Real Gettargetentropy(void) {return target_entropy;}
		// desired entropy
	Real Getentropy(void) {return entropy;}	// actual entropy
	BOOL Getgeneratorflag(void) {return generatorflag;}
	Real Getfailstripe(void) {return fail_stripe;}
	Real Getscore(){return score;}
	Real Getentropybonus(void) {return entropy_bonus;}
//----------------CA-Fourier Accessors-----------------------
	int Get_tpviewmode(void) {return tp_viewmode;}
	int Get_tpviewtype(void) {return tp_viewtype;}
	int Get_tpapproxtype(void) {return tp_approxtype;}
	BOOL Get_fourierflag(void) {return fourierflag;}
	int Get_numofterm(void) {return numofterm;}
	BOOL	Get_graphflag(){return tp_graphflag;}
	BOOL	Get_approxflag(){return tp_approxflag;}
	BOOL	Get_cosineflag(){return tp_cosineflag;}
	BOOL	Get_sineflag(){return tp_sineflag;}
	BOOL	Get_spectrumflag(){return tp_spectrumflag;}
//==============MUTATORS=====================================
	void Settype(int newtype);
	void Setviewmode(int newmode);
	void Setshowmode(int newmode){showmode = newmode;}
	void Set_dx_lock(BOOL l);
	void Setshowvelocity(int newshowvelocity){showvelocity = newshowvelocity;}
	void Setgeneratorflag(BOOL onoff);
//-------------CA-Seed Mutators-----------------
	void InitSeed();
		/* This sets all cells in all rows to zero. This is called
		initially from the constructor */
	void Seed(){FourierSeed();}; //Choose default Seed behaviour here.
	void RandomSeed();
		/* this seeds the source_row with random numbers
		between 0 and states - 1.  It is called in Locate at startup,
		after source_row is allocated.  Let each Wavecell.intensity and
		Wavecell.velocity range from -MAX_INTENSITY to +MAX_INTENSITY,
		and let the velocity be 0.0*/
	void FourierSeed();
		/* this seeds the source_row with fourier sums.*/
	void Sineseed();
		/* This seeds the source row with a periodic number pattern.
		Used especially for waves, where it seeds intentsity as a sine
		wave and velocity as the corresponding derivative cosine wave.*/
	void Oneseed();
		/* this seeds the source_row with all zeroes except for a
		single one in the center.  Set all the Wavecell's intensity
		and velocity to 0.0, except the central one, which gets a
		random intensity between -MAX_INTENSITY to +MAX_INTENSITY*/
	void Zeroseed();
		/* seeds with all zeros to calm down so you can see the
		generator*/
//---------CA-Color mutators---------------------
	void Setcolortable();
		/* Depending on the number of states, we set color values.*/
	void Setmonochromecolortable();
	void Setoldcolor();
		/* Try to get the good old 16 values of Rudy's color table */
	void SetSamecolortable(COLORREF *color, int bc);
	void SetColors(); // 2-24-96 DDC
	void Change_band_count(int updown);
	void Set_monochromeflag(int onoff);
	void Set_band_count(Real band);
//----------CA-Tweak Param mutators-----------------------
	void BumpTweakParam(int tweakParamType, Real updown);
	void SetTweakParam(TweakParam* t, Real val);
	void SetTweakParam(int tweakParamType, Real updown);
	void RandomizeTweakParam(TweakParam* t, Real valMean, Real valVariance);
	void Set_phase(Real p);
//----------CA-Digital Mutators----------------
	void Lambdalookup();
		// Sets a new random lookup with lambda percent non-zero values.
	void Changeradiusandstates(int newradius = 1, int newstates = -1);
	// if newradius or newstates == -1, then that part is unchanged
		/* These reset radius, states, nabesize, nabeoptions, and
		mask, frees the old lookup, and allocates a new one and
		fills it.  We need the combined form for copying CAs so that
		a legit copy isn't rejected because e.g. the radius looks too
		big before you notice the new state makes it ok. */
	void Symmetrize(); /* Make the lookup table summetric */
	void Setlambda(Real newlambda);
	void Changelambda(Real newlambda);
		/* Increment or decrement lambda if updown is, respectively
		1 or -1.  In the range 0.1 to 0.9, change by adding or
		subtracting 0.1.  Out of that range chagne by halving the
		distance to, respectively, 1.0 or 0.0.  Then change the current
		lookup table's density of nonzero entries to the new lambda.*/
//--------------------CA-Digital Reversible Mutator---------------
	void Reverse(); /* swap the past_row entries with the
		source_row entries.  This makes a TYPE_CA_REVERSIBLE
		rule run backwards.  It works like this.  In a rule
		like this,
		Target = f(Source) - Past, so also
		Past = f(Source) - Target.  At the end of a call to
		ReversibleUpdate the buffers get rolled forward.  So
		if I start with Past0 and Source0 and compute Target0,
		then Past1 is Source0 and Source1 is Target0.  If I
		exchange the entries in these two to get Source2 =
		Past1 and Past2 = Source1, then when I update
		Target1 = f(Source2) - Past2, I am really doing
		Target1 = f(Past1) - Source1, which is
		Target1 = f(Source0) - Target0, which  is Past0.
			 TODO: make this work for the alt waves!*/
//---------------------CA-Wave mutators-------------------------------
	void Set_startsmoothsteps(int s);
	/*Needed to split Get_spring next one into multiple rocker use.
		TODO it would be better to add separate methods*/
	void Reset_smoothsteps(){_smoothsteps = _startsmoothsteps;}
	void Set_justloadedcells(BOOL onoff){_justloadedcells = onoff;}
	void One_smoothsteps(){_smoothsteps = MINSMOOTHSTEPS;}
	void Change_phase(int updown);
//---------------CA-Diverse Cell Mutators---------------
	void Smooth_variance();
//------------CA-Evolution Mutators---------------------
	void Setfailstripe(int updown);
		// Penalty for failing the stripe test
	void Setwrapflag(BOOL onoff);
	void Setblankedgeflag(BOOL onoff);
	void Setentropyflag(BOOL onoff) {entropyflag = onoff;
		 if(entropyflag)Resetfreq();}
	void Setentropybonus(int updown);
		// Bonus for getting an entropy close to target_entropy
	void CopyCA(CA *goodCA);
		/* Copy everything from goodCA. Wave stuff, source_row, ... */
	void Mutate(Real mutation_strength);
		/* mutation_strength between 0.0 and 1.0.  Mutate that
		proportion of the table's entries*/
	void Install_Ramp(int paramcode, Real rampvalue);
//----------------CA-Fourier Mutators-----------------------
	void Set_tpviewmode(int new_viewmode) {tp_viewmode = new_viewmode;}
	void Set_tpviewtype(int new_viewtype) {tp_viewtype = new_viewtype;}
	void Set_tpapproxtype(int new_atype){tp_approxtype = new_atype;}
	void Set_fourierflag(BOOL num) {fourierflag = num;}
	void Set_TP(void);
	void Set_TempTP(void) {temp_test_point = test_point;}
	void Change_TempTP(int);
	void Set_numofterm(int nt) {numofterm = nt;}
	void Change_numofterm(int);
	void Reset_tp_array(void);
	void  Reset_fourier_array(void);
	void	Reset_tp_all(void);
	void	Change_approxflag(){tp_approxflag = 1 - tp_approxflag;}
	void	Change_cosineflag(){tp_cosineflag = 1 - tp_cosineflag;}
	void	Change_sineflag(){tp_sineflag = 1 - tp_sineflag;}
	void	Change_spectrumflag(){tp_spectrumflag = 1 - tp_spectrumflag;}
	void	Change_graphflag(){tp_graphflag = 1 - tp_graphflag;}
//=========================METHODS=================
	void Full_adjust_params_for_type(); /* Fix various things to be stable.
		This gets called after SetType in user RandomizeSettype.  This
		 makes problems in loading files.  I'll use this when user
		hand-changes a type also.*/
	void Basic_adjust_params_for_type(); /* Fix the minimum.  OK
		for loading files.*/
	void Locate(int itile_number = 0,int dmaxx = 0, int dmaxy = 0,
		int CA_count_per_edge = 1);
		/*tile_count tells how many CAs you want on screen, and
		itile_number tells which position this CA is in.  itile_number
		lies between 0 and tile_count-1.  Locate sets minx,maxx,miny,
		maxy, and intializes row_number to miny.Locate sets horz_count to
		maxx + 1 - minx, which is the and allocates source_row and
		target_row.*/
	void SyncRows();
		/* Copy source_row[0 to horz_count] into rowbuffer[0],
		copy past_row[0 to horz_count] into rowbuffer[MEMORY - 1],
		copy wave_source_row[0 to horz_count] back to start_wave_row
		this only does something if source and target are flipped
		from their original positions.
		This is because when we seed in the beginning, we only seed
		rowbuffer[0] with the 'whole' line.
		We need to do this when we resize, because we need the extra
		values stored in rowbuffer[0]. File save also uses this because
		we want to save the whole line */
	int Getviewmode(){return viewmode;}
  	void Touch_CA(int cx, int cy, unsigned int sender); /* Impliments users ability to
   	touch a CA at location cx,cy*/
//----------CA Digital helpers---------------------
	void Computeactual_lambda();	// Find a CAs actual lambda level
//----------------------CA Wave helpers-------------------------------
	void SmoothAverageStretch1D();
	void SmoothAverageStretch2D();
	void SmoothEdge1D();
	void SmoothEdge2D();
	void Adjust_acceleration_multiplier(); //Adjusts a bunch of params
//--------------CA-Oscillator Helper--------------------------------------
#ifdef RUNGE_KUTTE
	/* RUNGE-KUTTE helper for Oscillator , and Oscillator, no longer used. */
	DerivPair OscDerivs(Real intensity, Real velocity, Real dt);
	DerivPair DiverseOscDerivs(Real intensity, Real velocity, Real dt,
		Real massparam, Real frictionparam, Real springparam);
#endif //RUNGE_KUTTE
//------------------CA 1D Update functions public------------------------
	void (CA::*UpdateFunction)(HDC hdc);  /* This is a member function
		pointer as decribed to me by Horstmann, with a reference to
		 "Coplein's book on advanced C++ idioms".*/
	void Update(HDC hdc){(this->*UpdateFunction)(hdc);} //CRASH here once,
	//suggests that sometimes UpdateFunction isn't getting initialized anymore.
	; //Define in ca.cpp as a switch on ca_type.
	void (CA::*UpdateCell)(int, int, int);
		/* This is a member function pointer used by Wave functions
		to update the velocity and intinsity*/
	void (CA::*UpdateCell2)(int, int, int, int, int);  //2nd order or 2D
		/* This is a member function pointer used by Wave functions
		to update the velocity and intinsity*/
	void Show(HDC hdc); /* This holds the code common to the STandard
		Updates and the Wave Updates, the show & stripe stuff, also
		the stuff dealing with wire mode. */
	void StandardUpdate(HDC hdc);
		/* this is the heart of the algorithm.  You update the values
		of target_row by successively converting each cell's neighborhood
		into a bit-assembled nabe code number.  When states is 2, you use
		repeated calls of nabe <<= 1, nabe&= mask, and
		nabe |= source_row[i], and then compute target_row[i] as
		lookup[nabe].  When target_row is ready, you swap its pointer
		with source_row.  And then you increment row_number.*/
	void ReversibleUpdate(HDC hdc);
		/* This is the same as the CA_standard Update except that
		while before, you have NewC = lookup(C_nabe), you now let
		NewC = ( f(C_nabe) - OldC ) MOD (states).*/
	void WaveUpdate(HDC hdc);
		/* The wave update method is to compute acceleration as
		(2* C_intensity) - (L_intensity + R_intensity),
		to then do velocity += _dt * acceleration,
		and then do intensity += _dt * velocity.*/
	void WaveUpdate2(HDC hdc);
		/* Two neighbors either side.*/
	void WaveUpdateStep(HDC hdc);
		/* This holds the code common to WaveUpdate and WaveUpdate2, all
		the stepping stuff.*/
	//first order time derivative
	void Smooth(int, int, int);
	void HeatInt1(int, int, int);
	void HeatInt2(int, int, int, int, int);
	//wrong wave
	void WaveVelInt1(int, int, int);
	void WaveVelInt2(int, int, int, int, int); // five nabe
	//Right wave
	void AltWaveVelInt1(int, int, int);
	void Oscillator(int, int, int);
	void DiverseOscillator(int, int, int);
	void AltOscillatorWave(int, int, int);
	void AltDiverseOscillatorWave(int, int, int);
	//---------------CA-Nonlinear wave UPdate functions-------
	//alternate algorithms for second order time derivatives
	void UlamWave(int, int, int);
	void StableUlamWave(int, int, int);
	void AltUlamWave(int, int, int);
	void AltStableUlamWave(int, int, int);
//--------------CA 2D update functions ----------------------
	void (CA::*UpdateCell2D)(index2D_type, index2D_type, index2D_type,
		index2D_type, index2D_type);  //2nd order or 2D
		/* This is a member function pointer used by Wave functions
		to update the intinsity*/
	index2D_type index(int x, int y)
		{return (index2D_type)y*CX_2D + (index2D_type)x;}
	void WaveUpdate2D(HDC hdc);
	void Wave2D(index2D_type, index2D_type, index2D_type,
		index2D_type, index2D_type);
	void Heat2D(index2D_type, index2D_type, index2D_type,
		index2D_type, index2D_type);
	void Smooth2D(index2D_type, index2D_type, index2D_type,
		index2D_type, index2D_type);
	void WaveUpdateStep2D(HDC hdc);
//-------------------CA DLL Update Methods -----------------------------------
	BOOL SelectUserRuleName(HWND);
	BOOL SelectUSERRULE(); //Sets lpfnUSERRULE to match USERRULE in library.
	//SDLLRule is a rule like UlamWave, etc., that you use for CA::*UpdateCell
	void DLLRule(int, int, int); //calls (*pbfnUSERRULE)(this,l,c,r);
	//Need a "friend" declaration so the USERRULE module can see CA privates.
	friend DllExport USERRULE(CA FAR *, int, int, int);
//-------------CA-Evolution Methods-------------------------------------
	void Resetscore() {score = 0;}	// CA's score
	void Prepareforentropy();
		/* This gets freqlookup ready according as to whether
		entropyflag is 0 for off or 1 for on */
	void Changeentropy(Real newentropy);
		/* increment or decrement the target_entropy */
	void Become_child_of(CA *mom, CA *dad);
		/* Adopt mom's states and radius.  Make your lookup be
		a combination of mom's and dad's.  Use the part of mom's
		and part of dad's.  If dad's is shorter, copy his whole into
		the start of mom's lookup.*/
	void Entropy();
		/* Use freqlookup to compute the entropy if the entropyflag is
		on.  This is called everytime that cellcount passes 64K.  The
		fitness is set according to the entropy compared to
		the target_entropy.*/
	void Resetfreq();
		/* set cellcount to zero and zero out the freqlookup table
		if entropyflag is on*/
	void Avoidstripes(); // If the CA has a reoccuring pattern, Seed it.
//----------------CA-Fourier Methods-----------------------
	void Fourier_init(void);
	void SFT(void);						// slow fourier transform
	void FFT(void);						// fast fourier transform
	void SineSFT(int, int, int);
	void CosineSFT(int, int, int);
	void BothSFT(int, int, int);
	void Showpointgraph(HDC);			// show the graph
	int  Screen_yvalue(Real, Real, int, int);// translate to screen y_coord
};

//===================================CALIST CLASS====================
class DLLlist;

class CAlist
{
	friend class CA;
private:
//-----------------CAlistBase private--------------------------------
	HWND hwnd; //AUTOSET.  Where I live
	int count; // LOADSAVE. Number of CA objects in the list.
	CA *list[MAX_CAS]; //AUTOSET. Pointer list to the CAs
	CA *focus; //AUTOSET. This is the CA with the box around it.
	DLLlist *dll_list;//AUTOSET.
	WindowBitmap *WBM; /*AUTOSET WindowBitmap. Put pixels here,
     	 then bitblt all to screen*/
	BOOL zoomflag; //LOADSAVE. If this is on, we only show the focus CA.
	int CA_count_per_edge; //AUTOSET. length & height of displayed cells
	int sleep; /* AUTOSET. Used when the window is minimized. If sleep = 1,
		then update and show will not work. */
	BOOL _justloadedcells; /*Set this to TRUE when you have just loaded a
		*.CAS file which loads individual cell values, this is to prevent the
		call to smooth which would wipe out the values */
//--------CAlist Color private----------------------------
	int _numcolor; //AUTOSET
	HPALETTE _hpal; //AUTOSET
	LOGPALETTE *_plgpl; //AUTOSET
//--------CAlist Evolution private----------------------------
	Real mutation_strength; //LOADSAVE. Amount to mutate 0 to 1.
	int best_list[MAX_CAS]; /* AUTOSET.
     	This has all the CA position numbers sorted
		out by the best score. Set by Sortfittness() */
	BOOL mutateflag; // LOADSAVE. Mutate all CAs at breed cycle time?
	BOOL breedflag; 	// LOADSAVE. Enable breed cycle?
	unsigned int breedcycle; // LOADSAVE. We breed after this many cycles
	int breedcycle_count; // LOADSAVE. breed cycle counter
	int evolveflag; /* LOADSAVE. Methods to breed. 0, 1, 2 are
		Zap, Clone, and Sex. */
	BOOL stripekillflag; /* LOADSAVE. If 1, then we do a lamndalookup
     	right away on somebody who fails the stripe test, otherwise
			 we add the penalty to his score */
	BOOL stripeseedflag; //LOADSAVE. seed if stripe fail
	// palette stuff added by JC 12/95
//--------CAlist Fourier private----------------------------
	int tp_dowhich; //fourier
//----------CAlist Compress private-------------------------
   typedef struct hcode {  //compress
		 char ch1;
		 int  bit_code;
		 int  bit_count;
	 } Hcode;
	 typedef struct tree_node {    //compress
	 char code;
	 tree_node *LEFT;
	 tree_node *RIGHT;
	 } NODE;
public:
//===============CAlist Constructors================
	CAlist(HWND myhwnd, int icount); //constructor
	~CAlist(); //destructor
//===============CAlist Accessors================
	int Count(){return count;} //listlength
	CA *FocusCA(){return focus;}	// This is a pointer to the focus CA
	CA *GetCA(int i) {return list[i];}  // returns ith CA
	int Getdimension(){return list[0]->Getdimension();}
	int Getshowvelocity(){return FocusCA()->showvelocity;}
	BOOL Getzoomflag() { return zoomflag; }
	int GetSleep(){return sleep;} // Use with IDM_PAUSE in capow.cpp
	BOOL Get_justloadedcells(){return _justloadedcells;}
//---------------CAlist Color Accessors------------------
	int numcolor() { return _numcolor; }
	HPALETTE hpal() { return _hpal; }
	LOGPALETTE * logpal(){return _plgpl;}
//---------------CAlist Evolution Accessors------------------
	int Getbreedingsteps() {return breedcycle;} // Breed after this many cycles
	BOOL Getbreedflag() {return breedflag;}
		// Breed cycle enabled?
	BOOL Getstripekillflag(void) { return stripekillflag; }
	BOOL Getstripeseedflag(void) { return stripeseedflag; }
	Real Getmutation() {return mutation_strength;}
		/* Mutation strength is a percentage 0.0 to 1.0 of how much
		of the lookup you wanted mutated */
	BOOL Getmutateflag() {return mutateflag;}
		// Mutate at Breed cycle?
	int Getevolve(void) {return evolveflag;}
		/* Sets and Gets the ca's evolve flag for use by Cycle
		0 - Zap, 1 - Clone, 2 - Breed */
//===============CAlist Mutators================
	int Changecount(int icount); /* Used by expand and shrink. Changes the
		number of CAs shown on the screen to icount */
	int Bumpcount(int updown); /* Adds or takes away a whole CA_count_per_edge of
		CAs. e.g. 3x3 to 4x4 or vise versa. Updown is 1 or -1 */
	void SetCAType(CA *target, int newtype, BOOL fixflag);
		/* Changes the type of the CA.  If fixflag is TRUE, you
		also call the CA::Full_adjust_params_for_type to make the
		params be nice looking.*/
	void SetAllType(int newtype, BOOL fixflag);
		/* Changes the type of all the cas, fixflag as with SetCAType*/
	void Setviewmode(int newmode); //Do all in list.
	void Setwrapflag(BOOL onoff);	/* 0 for off, 1 for on */
	void Set_startsmoothsteps(int s);
	void Setshowmode(int newmode);
	void Setgeneratorflag(BOOL onoff);	/* 0 for off, 1 for on */
	void Setblankedgeflag(BOOL onoff);
	void Set_dx_lock(BOOL onoff);
	void Reset_smoothsteps();
	void One_smoothsteps();
	void Setshowvelocity(int newshowvelocity, short focusflag);//new way
	int Setfocus(HDC hdc, CA *newfocus);	/* Change focus CA */
	void SetWindowBitmap(WindowBitmap *myWBM);	/* Initializes the Bitmap
		pointer for use by calist and each CA gets a copy also*/
	void SetSleep(BOOL onoff); // 1 - Window is minimized, dont update or show
	void ToggleSleep(){sleep ^= 1;}//Ditto
	void Set_justloadedcells(BOOL onoff){_justloadedcells = onoff;}
	void SyncRows(); //Calls this on the members.
//---------------CAlist Color Mutators------------------
	void Set_monochromeflag(int onoff);
	void Set_band_count(Real band);
	void Setnumcolor(int newnumcolor);
	void SetDefaultLogPalette();
	void SetRandomLogPalette(COLORREF *_anchor_color, int band_count);
	void SetMonochromeLogPalette();
   void SetColors(); //NEW 3-1-96 DDC
	void Setcolortable();
//------------------ CAlist TweakParam mutator------------------------
	void BumpCATweakParam (int tweakParamType, int updown);
   void SetTweakParam (int tweakParamType, Real val);
//--------------------CAlist Digital mutator-------------------------
	void Changelambda(Real newlambda); // Change all CA's lambda
   void Setlambda (Real newlambda);  // Set newlambda to all CAs
//------------------CAlist Wave mutator-------------------------------
	void Change_band_count(int updown);
	void Change_phase(int updown);
//----------------------CAlist Fourier mutators---------------------
	void Reset_tpgraph();
	int Get_tpdowhich(void) {return tp_dowhich;}
	void  Set_tpdowhich(int new_dowhich) {tp_dowhich = new_dowhich;}
//---------------CAlist Evolution Mutators------------------
	void Setentropy(BOOL onoff); /* 0 for off, 1 for on */
	void Setstripekillflag(BOOL onoff) { stripekillflag = onoff; }
	void Setstripeseedflag(BOOL onoff) { stripeseedflag = onoff; }
	void Setmutation(int updown);
	void Setmutateflag(BOOL imutate) {mutateflag = imutate;}
	void Setbreedflag(BOOL ibreed) {breedflag = ibreed;}
	void Setfailstripe(int updown); // fail stripe penalty
	void Setentropybonus(int updown); // bonus for a close entropy
	void Setbreedcycle(int updown); // When do we Breed? 1 or -1
	void SetEvolve(BOOL newevolve) {evolveflag = newevolve;}
//===============CAlist Methods================
	void Show(HDC hdc, const PAINTSTRUCT &ps);
	void Update_and_Show(HDC hdc);  /*Update all active CAs and show the
		updates in a rapid way.*/
	void Locate(int dmaxx, int dmaxy); /*This gnarly method uses count and
		dp to figure out where to tell the list[i] CAs to put their
		minx, maxx, miny, maxy.*/
	void Randomize(); //Randomizes type_ca, state, radius
	void Touch_CA(int cx, int cy, unsigned int sender); /* Impliments users ability to
	touch a CA at location cx,cy*/
	int Zoom(int inout); /* 1 for in (1 CA), -1 for out (the active CAs).
		When you do Zoom(1), you move focus to the CA you get to see.
		Return a 0 if you're repeating a zoom.*/
 	CA *Getfocus(int cursorx, int cursory);	/* return pointer to the CA
		that is positioned on screen at cursorx,cursory */
	void Boxfocus(HDC hdc, COLORREF color); //Draw a box around focus CA.
	void DrawDivider(HDC hdc);
//------------CAlist Seed Methods----------------------
	void Seed(); //Default seed with CA::Seed
	void RandomSeed(); //Seed all active CAs.
	void FourierSeed(); //Seed all active CAs.
	void Oneseed();	// Call each CA's oneseed
	void Zeroseed(); //Call each CA's zeroseed
	void Sineseed();	// Call each CA's sineseed
//------------CAlist Evolution Methods----------------------
	void Evolve();/* After each breed cycle this is called
		depending what evolveflag is set to this can happen:
		0-zap - mutates everybody
		1-clone - clones the best ones and mutates all
		2-breed - breeds the best ones and mutates all */
	void Sortfitness(); /* This uses the best_list[] array to sort
		all the CAs by their score.  The best_list[] array holds
		CA numbers */
	Real Mutation_strength(){return mutation_strength;} /* How much to
		mutate 0 to 1.0 */
	void Changeentropy(Real newentropy);	// Changes the target_entropy
	void Mutate();	// Mutate each CA by mutation_strength
	void Breed();	// Makes all the CAs a child of the focus and top left
	void Copymutate();	// Copys a mutation of the focus to each CA
	void Resetscores();	// Sets all CA scores to 0
	void CopyCA(CA *ca1, CA *ca2);
		/* Copies ca1 to ca2 */
//------------CAlist Compress Methods----------------------
	int  compress(ifstream &infile,ofstream &outfile);
   int  uncompress(ifstream &infile,ofstream &outfile);
//------------CAlist Experiment Methods----------------------
	void Install_Ramp(int paramcode, Real *rampvalue);
	void CheckRange (int paramcode, Real minmax[]);
//--------CAlist File Methods-------------------------------
	void Saveindividual(char *filename, CA *target); // Save focus CA
	void Saveall(char *filename, BOOL auto_overwrite = FALSE); // Save entire experiment
     void Load_Individual(char* filename, CA *target); // Load on focus CA
	BOOL Loadall(char* filename, BOOL startup=FALSE); // Load entire experiment
//If startup then you don't call a WM_SIZE from this.
	//--------------------CAlist Digital methods-------------------------
	void Changeradiusandstates(int newradius = -1, int newstates = -1);
	// if newradius or newstates == -1, then that part is unchanged
	void Lambdalookup(); // Call each CA's lambda lookup.
	void Symmetrize(); // Call each CA's symmetrize
//------------------CAlist Reversible public-------------------------
	void Reverse(); // Call each CA's Reverse
//----------------------CAlist DLL methods-----------------------
	BOOL SelectUserRuleName(HWND);
	FPCELLUPDATE GetUserRulePtr(char *name); // typedef of FPCELLUPDATE is in types.h 
};

#endif CA_HPP
